/*----------------------------------------------------------------
 *sprite.c -- Simplified sprite version.
 *----------------------------------------------------------------
 */


#include <allegro.h>		
#include <stdlib.h>
#include "error.h"	

static int Down = 0;
static int Right = 0;
static int Up = 0;
static int Left = 0;
static int Cheering = 0;
static int NoKeys;

static int PosX = 400;
static int PosY = 300;
static int quit = 0;
static int LeftMax = 6;
static int RightMax = 6;
static int UpMax = 6;
static int DownMax = 6;

static int MaxX = 600;
static int MinX = 100;
static int MaxY = 400;
static int MinY = 100;

static BITMAP *CurrentGraphic;
static BITMAP *Standing;
static BITMAP *Stand[4];
static BITMAP *Sleep[2];
static BITMAP *Wake[3];
static BITMAP *Cheer[2];
static BITMAP *WalkLeft[6];
static BITMAP *WalkUp[6];
static BITMAP *WalkRight[6];
static BITMAP *WalkDown[6];
static BITMAP *Surprise;
static BITMAP *Erase;

static PALETTE palette;	

void sprite_init() 
{
	CurrentGraphic = load_bitmap ("sprite/stand_f.pcx", palette);
	if (!CurrentGraphic) error ("Unable to load current graphic",NULL); 

	Standing = load_bitmap ("sprite/stand_f.pcx", palette);
	if (!Standing) error ("Unable to load current graphic",NULL); 

	Surprise = load_bitmap ("sprite/surprise.pcx", palette);
	if (!Surprise) error ("Unable to load current graphic",NULL); 

	Erase = load_bitmap ("sprite/erase.pcx", palette);
	if (!CurrentGraphic) error ("Unable to load current graphic",NULL); 

	Stand[0] = load_bitmap ("sprite/stand_f.pcx", palette);
	if (!Stand[0]) error ("Unable to load standing 0 graphic",NULL); 
	Stand[1] = load_bitmap ("sprite/stand_b.pcx", palette);
	if (!Stand[1]) error ("Unable to load standing 1 graphic",NULL); 
	Stand[2] = load_bitmap ("sprite/stand_l.pcx", palette);
	if (!Stand[2]) error ("Unable to load standing 2 graphic",NULL); 
	Stand[3] = load_bitmap ("sprite/stand_r.pcx", palette);
	if (!Stand[3]) error ("Unable to load standing 3 graphic",NULL); 

	Sleep[0] = load_bitmap ("sprite/sleep_1.pcx", palette);
	if (!Sleep[0]) error ("Unable to load sleeping 1 graphic",NULL); 
	Sleep[1] = load_bitmap ("sprite/sleep_2.pcx", palette);
	if (!Sleep[1]) error ("Unable to load sleeping 2 graphic",NULL); 

	Wake[0] = load_bitmap ("sprite/wake_1.pcx", palette);
	if (!Wake[0]) error ("Unable to load waking 1 graphic",NULL); 
	Wake[1] = load_bitmap ("sprite/wake_2.pcx", palette);
	if (!Wake[1]) error ("Unable to load waking 2 graphic",NULL); 
	Wake[2] = load_bitmap ("sprite/wake_3.pcx", palette);
	if (!Wake[2]) error ("Unable to load waking 3 graphic",NULL); 

	Cheer[0] = load_bitmap ("sprite/cheer_1.pcx", palette);
	if (!Cheer[0]) error ("Unable to load cheering 1 graphic",NULL); 
	Cheer[1] = load_bitmap ("sprite/cheer_2.pcx", palette);
	if (!Cheer[1]) error ("Unable to load cheering 2 graphic",NULL); 

	WalkLeft[0] = load_bitmap ("sprite/walk_l1.pcx", palette);
	if (!WalkLeft[0]) error ("Unable to load walking left 1 graphic",NULL); 
	WalkLeft[1] = load_bitmap ("sprite/walk_l2.pcx", palette);
	if (!WalkLeft[1]) error ("Unable to load walking left 2 graphic",NULL); 
	WalkLeft[2] = load_bitmap ("sprite/walk_l3.pcx", palette);
	if (!WalkLeft[2]) error ("Unable to load walking left 3 graphic",NULL); 
	WalkLeft[3] = load_bitmap ("sprite/walk_l4.pcx", palette);
	if (!WalkLeft[3]) error ("Unable to load walking left 4 graphic",NULL); 
	WalkLeft[4] = load_bitmap ("sprite/walk_l5.pcx", palette);
	if (!WalkLeft[4]) error ("Unable to load walking left 5 graphic",NULL); 
	WalkLeft[5] = load_bitmap ("sprite/walk_l6.pcx", palette);
	if (!WalkLeft[5]) error ("Unable to load walking left 6 graphic",NULL); 

	WalkUp[0] = load_bitmap ("sprite/walk_b1.pcx", palette);
	if (!WalkUp[0]) error ("Unable to load title graphic",NULL); 
	WalkUp[1] = load_bitmap ("sprite/walk_b2.pcx", palette);
	if (!WalkUp[1]) error ("Unable to load title graphic",NULL); 
	WalkUp[2] = load_bitmap ("sprite/walk_b3.pcx", palette);
	if (!WalkUp[2]) error ("Unable to load title graphic",NULL); 
	WalkUp[3] = load_bitmap ("sprite/walk_b4.pcx", palette);
	if (!WalkUp[3]) error ("Unable to load title graphic",NULL); 
	WalkUp[4] = load_bitmap ("sprite/walk_b5.pcx", palette);
	if (!WalkUp[4]) error ("Unable to load title graphic",NULL); 
	WalkUp[5] = load_bitmap ("sprite/walk_b6.pcx", palette);
	if (!WalkUp[5]) error ("Unable to load title graphic",NULL); 

	WalkRight[0] = load_bitmap ("sprite/walk_r1.pcx", palette);
	if (!WalkRight[0]) error ("Unable to load title graphic",NULL); 
	WalkRight[1] = load_bitmap ("sprite/walk_r2.pcx", palette);
	if (!WalkRight[1]) error ("Unable to load title graphic",NULL); 
	WalkRight[2] = load_bitmap ("sprite/walk_r3.pcx", palette);
	if (!WalkRight[2]) error ("Unable to load title graphic",NULL); 
	WalkRight[3] = load_bitmap ("sprite/walk_r4.pcx", palette);
	if (!WalkRight[3]) error ("Unable to load title graphic",NULL); 
	WalkRight[4] = load_bitmap ("sprite/walk_r5.pcx", palette);
	if (!WalkRight[4]) error ("Unable to load title graphic",NULL); 
	WalkRight[5] = load_bitmap ("sprite/walk_r6.pcx", palette);
	if (!WalkRight[5]) error ("Unable to load title graphic",NULL); 

	WalkDown[0] = load_bitmap ("sprite/walk_f1.pcx", palette);
	if (!WalkDown[0]) error ("Unable to load title graphic",NULL); 
	WalkDown[1] = load_bitmap ("sprite/walk_f2.pcx", palette);
	if (!WalkDown[1]) error ("Unable to load title graphic",NULL); 
	WalkDown[2] = load_bitmap ("sprite/walk_f3.pcx", palette);
	if (!WalkDown[2]) error ("Unable to load title graphic",NULL); 
	WalkDown[3] = load_bitmap ("sprite/walk_f4.pcx", palette);
	if (!WalkDown[3]) error ("Unable to load title graphic",NULL); 
	WalkDown[4] = load_bitmap ("sprite/walk_f5.pcx", palette);
	if (!WalkDown[4]) error ("Unable to load title graphic",NULL); 
	WalkDown[5] = load_bitmap ("sprite/walk_f6.pcx", palette);
	if (!WalkDown[5]) error ("Unable to load title graphic",NULL); 

	if (set_gfx_mode (GFX_AUTODETECT, 640, 480, 0, 0) < 0)
		error ("Unable to set video mode", allegro_error);
	set_palette (palette);	
        clear_to_color(screen, makecol(255, 255, 255));	
}

void CheckPosition()
{
 if( PosX > MaxX )
 {
  PosX = MaxX;
 }
 if( PosX < MinX )
 {
  PosX = MinX;
 }
	
 if( PosY > MaxY )
 {
  PosY = MaxY;
 }
 if( PosY < MinY )
 {
  PosY = MinY;
 }
}

void GoGoGo()
{
 NoKeys = 0;
 blit(Cheer[0], screen, 0, 0, PosX, PosY, 23, 35);
 rectfill(screen, 0, 0, 800, 30, makecol(255, 255, 255)); 
 int number = rand() % 13;
 switch(number) {
        case 0 :
                textout_centre(screen, font, "Pow boom! Straight to the moon!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        case 1 :
                textout_centre(screen, font, "Ra ra ra!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        case 2 :
                textout_centre(screen, font, "I AM the Lizard Queen!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        case 3 :
                textout_centre(screen, font, "Go me!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        case 4 :
                textout_centre(screen, font, "They're Greeaat!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        case 5 :
                textout_centre(screen, font, "Go Rice!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        case 6 :
                textout_centre(screen, font, "A rabbi and a hermaphrodite walk into a bar...", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        case 7 :
                textout_centre(screen, font, "Cactus!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        case 8 :
                textout_centre(screen, font, "Trying is the first step towards failure!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        case 9 :
                textout_centre(screen, font, "I am the man!  ...Or at least A man.", SCREEN_W/2, 10, makecol(0,0,0));
                break;
	case 10 :
                textout_centre(screen, font, "Social spending!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
	case 11 :
                textout_centre(screen, font, "Rated PG-13!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
	case 12 :
                textout_centre(screen, font, "Burninated!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        default :
                textout_centre(screen, font, "Yeah!", SCREEN_W/2, 10, makecol(0,0,0));
                break;
        }
 rest(600);
 vsync();
 blit(Cheer[1], screen, 0, 0, PosX, PosY, 23, 35);
 rest(800);
 vsync();
 blit(Standing, screen, 0, 0, PosX, PosY, 23, 35);
 rest(300);
 rectfill(screen, 0, 0, 800, 30, makecol(255, 255, 255)); 
}

int CheckKeys()
{
 Left = 0;
 Right = 0;
 Up = 0;
 Down = 0;
 Cheering = 0;

 if (key[KEY_F]) 
 {
	Standing = Stand[0];
	Cheering = 1;
	return 1;
 }
 else if (key[KEY_LEFT]) 
 {
	Left = 1;
	PosX -= 1;
	return 1;
 }
 else if (key[KEY_RIGHT]) 
 {
	Right = 1;
	PosX += 1;
	return 1;
 }
 else if (key[KEY_UP]) 
 {
	Up = 1;
	PosY -= 1;
	return 1;
 }
 else if (key[KEY_DOWN]) 
 {
	Down = 1;
	PosY += 1;
	return 1;
 }
 else  if (key[KEY_Q]) 
 {
	quit = 1;
	return 1;
 }
 else NoKeys++;

 return 0;
}

void WalkAnimate()
{
   blit(CurrentGraphic, screen, 0, 0, PosX, PosY, 23, 35);
   int j = 0;

   while( j < 4 )
   {
    if( Left == 1 )
    {
     PosX -= 1;
     CheckPosition();
     blit(Erase, screen, 0, 0, PosX +1, PosY, 23, 35);
    }

    if( Up == 1 )
    {
     PosY -= 1;
     CheckPosition();
     blit(Erase, screen, 0, 0, PosX, PosY+1, 23, 35);
    }

    if( Right == 1 )
    {
     PosX += 1;
     CheckPosition();
     blit(Erase, screen, 0, 0, PosX -1, PosY, 23, 35);
    }

    if( Down == 1 )
    {
     PosY += 1;
     CheckPosition();
     blit(Erase, screen, 0, 0, PosX, PosY -1, 23, 35);
    }
    blit(CurrentGraphic, screen, 0, 0, PosX, PosY, 23, 35);
    vsync();
    j++;
   }
}

void Nap()
{
 blit(Stand[2], screen, 0, 0, PosX, PosY, 23, 35); 
 textout_centre(screen, font, "Hmm..", SCREEN_W/2, 10, makecol(0,0,0));
 rest(500);
 blit(Stand[3], screen, 0, 0, PosX, PosY, 23, 35); 
 rest(1000);
 blit(Stand[2], screen, 0, 0, PosX, PosY, 23, 35); 
 rectfill(screen, 0, 0, 800, 30, makecol(255, 255, 255)); 
 textout_centre(screen, font, "*sigh*", SCREEN_W/2, 10, makecol(0,0,0));
 rest(1000);
 blit(Wake[0], screen, 0, 0, PosX, PosY, 23, 35); 
 rest(500);
 blit(Wake[1], screen, 0, 0, PosX, PosY, 23, 35); 
 rectfill(screen, 0, 0, 800, 30, makecol(255, 255, 255)); 
 textout_centre(screen, font, "I wonder where they went? ...", SCREEN_W/2, 10, makecol(0,0,0));
 rest(2000);
 blit(Wake[2], screen, 0, 0, PosX, PosY, 23, 35);
 rest(1000);
 int k = 2;
 clear_keybuf();
  rectfill(screen, 0, 0, 800, 30, makecol(255, 255, 255)); 
  textout_centre(screen, font, "*snore snore*", SCREEN_W/2, 10, makecol(0,0,0));
 while( !keypressed() )
 {

  if(keypressed())
  {
   CheckKeys();
  }
  blit(Sleep[k % 2], screen, 0, 0, PosX, PosY, 23, 35);
  rest(500);
  vsync();
  k++;
  if( k > 100 ) k = 2;
 } 

 blit(Wake[2], screen, 0, 0, PosX, PosY, 23, 35); 
 rest(300);
  rectfill(screen, 0, 0, 800, 30, makecol(255, 255, 255)); 
  textout_centre(screen, font, "Huh?  Wha? ...", SCREEN_W/2, 10, makecol(0,0,0));
 blit(Wake[1], screen, 0, 0, PosX, PosY, 23, 35);
 rest(500);
 blit(Wake[2], screen, 0, 0, PosX, PosY, 23, 35); 
 rest(500);
 blit(Wake[1], screen, 0, 0, PosX, PosY, 23, 35);
  rectfill(screen, 0, 0, 800, 30, makecol(255, 255, 255)); 
  textout_centre(screen, font, "Oh, there you are!", SCREEN_W/2, 10, makecol(0,0,0));
 blit(Surprise, screen, 0, 0, PosX, PosY, 23, 35);
 rest(1000);
 blit(Standing, screen, 0, 0, PosX, PosY, 23, 35);
  rectfill(screen, 0, 0, 800, 30, makecol(255, 255, 255)); 
 NoKeys = 0;
}

void animate() 
{
	CurrentGraphic = Stand[0];
        blit(CurrentGraphic, screen, 0, 0, PosX, PosY, 23, 35);

	int CurX = PosX;
	int CurY = PosY;
	int i = 0;

	while(quit == 0)
	{
	 vsync();

	 CheckPosition();
	 
	 if( i > 60 )
	 {
		i = 1;
	 }
	 else i++;
	 
	 CheckKeys();
	 if( NoKeys > 300 ) Nap();
	 if( Cheering == 1 ) GoGoGo();

 	 if( CurX != PosX || CurY !=PosY )
	 {
  	  NoKeys=0;
	  blit(Erase, screen, 0, 0, CurX, CurY, 23, 35);

	  if(Left == 1) 
	  {
	   CurrentGraphic = WalkLeft[(i % LeftMax)];
	   WalkAnimate();
	   Standing = Stand[2];
	  }

	  if(Right == 1)
	  {
	   CurrentGraphic = WalkRight[(i % RightMax)];
	   WalkAnimate();
	   Standing = Stand[3];
	  }

	  if(Up == 1) 
	  {
	   CurrentGraphic = WalkUp[(i % UpMax)];
	   WalkAnimate();
	   Standing = Stand[1];
	  }

	  if(Down == 1)
	  {
	   CurrentGraphic = WalkDown[(i % DownMax)];
	   WalkAnimate();
	   Standing = Stand[0];
	  }

 	  CurX = PosX;
	  CurY = PosY;
	 }
	 else
  	   blit(Standing, screen, 0, 0, PosX, PosY, 23, 35);
	}
}

static void init() 
{
	allegro_init();			
	install_timer();		
	install_keyboard();		
	initialise_joystick();
	install_mouse();
}


static void shutdown() 
{
 	allegro_exit();
}

static void Welcome()
{
 rect(screen, 95, 95, 624, 437, 255); 
 blit(CurrentGraphic, screen, 0, 0, PosX, PosY, 23, 35);
 text_mode(-1);
 textout_centre(screen, font, "Hello, my name is Crono! Try walking around with me!", SCREEN_W/2, 10, makecol(0,0,0));
 readkey();
  rectfill(screen, 0, 0, 800, 30, makecol(255, 255, 255)); 

}

static void sprite()			
{
	sprite_init();
	Welcome();
	animate();
}

int main (int argc, char *argv[])
{  
   srand( (unsigned int) time( NULL ));
   init();
   MIDI *the_music;	
   if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, argv[0]) != 0) {
      allegro_message("Error initialising sound system\n%s\n", allegro_error);
      return 1;		
   }

   the_music = load_midi("kitsune.mid");
   if (!the_music) {			
      allegro_message("Error reading MIDI file '%s'\n", "dear.mid");
      return 1;
   }
   play_midi(the_music, TRUE);	
   sprite();
   remove_sound();	
   destroy_midi(the_music);
   shutdown();
   return 0;			
}
END_OF_MAIN()
